/*****************************************************************************
* | File        :   LCD_Driver.c
* | Author      :   Waveshare team
* | Function    :   Electronic paper driver
* | Info        :
*----------------
* | This version:   V1.0
* | Date        :   2018-11-18
* | Info        :   
#
# Permission is hereby granted, free of UBYTEge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
******************************************************************************/
#include "LCD_Driver.h"

/*******************************************************************************
function:
  Hardware reset
*******************************************************************************/
static void LCD_Reset(void)
{
  DEV_Digital_Write(DEV_CS_PIN,0);
  DEV_Delay_ms(20);
  DEV_Digital_Write(DEV_RST_PIN,0);
  DEV_Delay_ms(20);
  DEV_Digital_Write(DEV_RST_PIN,1);
  DEV_Delay_ms(20);
}

/*******************************************************************************
function:
  Setting backlight
parameter :
    value : Range 0~255   Duty cycle is value/255
*******************************************************************************/
void LCD_SetBacklight(UWORD Value)
{
  DEV_Set_BL(DEV_BL_PIN, Value);
}

/*******************************************************************************
function:
    Write register address and data
*******************************************************************************/
void LCD_WriteData_Byte(UBYTE da) 
{ 
  DEV_Digital_Write(DEV_CS_PIN,0);
  DEV_Digital_Write(DEV_DC_PIN,1);
  DEV_SPI_WRITE(da);  
  DEV_Digital_Write(DEV_CS_PIN,1);
}  

 void LCD_WriteData_Word(UWORD da)
{
  UBYTE i=(da>>8)&0xff;
  DEV_Digital_Write(DEV_CS_PIN,0);
  DEV_Digital_Write(DEV_DC_PIN,1);
  DEV_SPI_WRITE(i);
  DEV_SPI_WRITE(da);
  DEV_Digital_Write(DEV_CS_PIN,1);
}   

void LCD_WriteReg(UBYTE da)  
{ 
  DEV_Digital_Write(DEV_CS_PIN,0);
  DEV_Digital_Write(DEV_DC_PIN,0);
  DEV_SPI_WRITE(da);
  DEV_Digital_Write(DEV_CS_PIN,1);
}

/******************************************************************************
function: 
    Common register initialization
******************************************************************************/
void LCD_Init(void)
{
  LCD_Reset();

  LCD_WriteReg(0x36);
	LCD_WriteData_Byte(0xC8);

  //************* Start Initial Sequence **********// 
	LCD_WriteReg(0xB0);		
	LCD_WriteData_Byte(0xC0); 
	LCD_WriteReg(0xB2);			
	LCD_WriteData_Byte(0x2F); 
	LCD_WriteReg(0xB3);		
	LCD_WriteData_Byte(0x03);
	LCD_WriteReg(0xB6);		
	LCD_WriteData_Byte(0x19); 
	LCD_WriteReg(0xB7);		
	LCD_WriteData_Byte(0x01);  
	
	LCD_WriteReg(0xAC);
	LCD_WriteData_Byte(0xCB);
	LCD_WriteReg(0xAB); 
	LCD_WriteData_Byte(0x0e);
		
	LCD_WriteReg(0xB4);	
	LCD_WriteData_Byte(0x04);
	
	LCD_WriteReg(0xA8);
	LCD_WriteData_Byte(0x19);

	LCD_WriteReg(0x3A);		
	LCD_WriteData_Byte(0x05); 

	LCD_WriteReg(0xb8);
	LCD_WriteData_Byte(0x08);
 
	LCD_WriteReg(0xE8);
	LCD_WriteData_Byte(0x24);

	LCD_WriteReg(0xE9);
	LCD_WriteData_Byte(0x48);

	LCD_WriteReg(0xea);	
	LCD_WriteData_Byte(0x22);
 			
	LCD_WriteReg(0xC6);
	LCD_WriteData_Byte(0x30);
	LCD_WriteReg(0xC7);
	LCD_WriteData_Byte(0x18);

	LCD_WriteReg(0xF0);
	LCD_WriteData_Byte(0x1F);
	LCD_WriteData_Byte(0x28);
	LCD_WriteData_Byte(0x04);
	LCD_WriteData_Byte(0x3E);
	LCD_WriteData_Byte(0x2A);
	LCD_WriteData_Byte(0x2E);
	LCD_WriteData_Byte(0x20);
	LCD_WriteData_Byte(0x00);
	LCD_WriteData_Byte(0x0C);
	LCD_WriteData_Byte(0x06);
	LCD_WriteData_Byte(0x00);
	LCD_WriteData_Byte(0x1C);
	LCD_WriteData_Byte(0x1F);
	LCD_WriteData_Byte(0x0f);

	LCD_WriteReg(0xF1); 
	LCD_WriteData_Byte(0X00);
	LCD_WriteData_Byte(0X2D);
	LCD_WriteData_Byte(0X2F);
	LCD_WriteData_Byte(0X3C);
	LCD_WriteData_Byte(0X6F);
	LCD_WriteData_Byte(0X1C);
	LCD_WriteData_Byte(0X0B);
	LCD_WriteData_Byte(0X00);
	LCD_WriteData_Byte(0X00);
	LCD_WriteData_Byte(0X00);
	LCD_WriteData_Byte(0X07);
	LCD_WriteData_Byte(0X0D);
	LCD_WriteData_Byte(0X11);
	LCD_WriteData_Byte(0X0f);

  LCD_WriteReg(0x21); 

  LCD_WriteReg(0x11); 

  LCD_WriteReg(0x29); 
} 

/******************************************************************************
function: Set the cursor position
parameter :
    Xstart:   Start UWORD x coordinate
    Ystart:   Start UWORD y coordinate
    Xend  :   End UWORD coordinates
    Yend  :   End UWORD coordinatesen
******************************************************************************/
void LCD_SetCursor(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD  Yend)
{ 
  	Ystart = Ystart + 1;
	  Yend = Yend + 1;
	  Xstart = Xstart + 2;
	  Xend = Xend +  2;

    //set the X coordinates
    LCD_WriteReg(0x2A);
    LCD_WriteData_Byte((Xstart >> 8) & 0xFF);
    LCD_WriteData_Byte(Xstart & 0xFF);
    LCD_WriteData_Byte(((Xend  ) >> 8) & 0xFF);
    LCD_WriteData_Byte((Xend  ) & 0xFF);

    //set the Y coordinates
    LCD_WriteReg(0x2B);
    LCD_WriteData_Byte((Ystart >> 8) & 0xFF);
    LCD_WriteData_Byte(Ystart & 0xFF);
    LCD_WriteData_Byte(((Yend  ) >> 8) & 0xFF);
    LCD_WriteData_Byte((Yend  ) & 0xFF);

  LCD_WriteReg(0x2C);
}

/******************************************************************************
function: Clear screen function, refresh the screen to a certain color
parameter :
    Color :   The color you want to clear all the screen
******************************************************************************/
void LCD_Clear(UWORD Color)
{
  UWORD i,j;    
  LCD_SetCursor(0,0,LCD_WIDTH-1,LCD_HEIGHT-1);
  for(i = 0; i < LCD_WIDTH; i++){
    for(j = 0; j < LCD_HEIGHT; j++){
      LCD_WriteData_Word(Color);
    }
   }
}

/******************************************************************************
function: Refresh a certain area to the same color
parameter :
    Xstart:   Start UWORD x coordinate
    Ystart:   Start UWORD y coordinate
    Xend  :   End UWORD coordinates
    Yend  :   End UWORD coordinates
    color :   Set the color
******************************************************************************/
void LCD_ClearWindow(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend,UWORD color)
{          
  UWORD i,j; 
  LCD_SetCursor(Xstart, Ystart, Xend-1,Yend-1);
  for(i = Ystart; i <= Yend-1; i++){                                
    for(j = Xstart; j <= Xend-1; j++){
      LCD_WriteData_Word(color);
    }
  }                   
}

/******************************************************************************
function: Set the color of an area
parameter :
    Xstart:   Start UWORD x coordinate
    Ystart:   Start UWORD y coordinate
    Xend  :   End UWORD coordinates
    Yend  :   End UWORD coordinates
    Color :   Set the color
******************************************************************************/
void LCD_SetWindowColor(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend,UWORD  Color)
{
  LCD_SetCursor( Xstart,Ystart,Xend,Yend);
  LCD_WriteData_Word(Color);      
}

/******************************************************************************
function: Draw a UWORD
parameter :
    X     :   Set the X coordinate
    Y     :   Set the Y coordinate
    Color :   Set the color
******************************************************************************/
void LCD_SetUWORD(UWORD x, UWORD y, UWORD Color)
{
  LCD_SetCursor(x,y,x,y);
  LCD_WriteData_Word(Color);      
} 
